# We need to loop through the spatial data of daily precipitation from 1979 to 2018 to find the variables during extreme precipitation event (EPE) days. Features like largest precipitation, mean precipitation, annual sum of precipitation, mean number of EPE days, total number of EPE days, 95th percentile value, contribution to total annual precipitation during EPE days

#Code

import xarray as xr
import numpy as np

# Define your spatial grid (replace with your actual latitudes and longitudes)
#lats = #define latitude values
#lons = #define longitude values

# Initialize empty lists to store calculated values
maxiv = []	# List for maximum EPE precipitation values
meanv = []	# List for mean EPE precipitation values
annsumv = []  # List for mean annual sum of precipitation from EPE events
countmean = []  # List for mean number of EPE days per year
countsum = []   # List for total number of EPE days
pert = []	# List for 95th percentile value
conpercv = []  # List for percentage contribution to total annual precipitation

# Loop through each spatial point
for i in lats:
	for j in lons:
    	# Sample data (replace with your actual precipitation data)
    	# ppnsp = daily precipitation values covering the area for the selected time (here, ERA5 hourly data)

    	# Select precipitation values at the current point and filter out low values
    	ppn1 = ppnsp.sel(latitude=i, longitude=j, method='nearest') * 1000
    	ppnd = ppn1[ppn1 > 0.02]

    	# Calculate the 95th percentile value
    	pert1 = np.percentile(ppnd, 95)

    	# Select EPE days based on the 95th percentile
    	epe1 = ppnd[ppnd > pert1]

    	# Calculate various statistics
    	maxi = epe1.values.max()  # Maximum value of EPE precipitation at that point
    	mean = epe1.values.mean()  # Mean value of EPE precipitation at that point
    	countm = epe1.groupby('time.year').count().mean().values  # Mean number of EPEs per year
    	counts = epe1.groupby('time.year').count().sum().values  # Total number of EPEs at that point
    	annsum = epe1.groupby('time.year').sum().mean()  # Mean of annual sum of precipitation from EPEs
    	totalsum = ppn1.groupby('time.year').sum(dim='time').mean()  # Mean annual sum of precipitation at that point
    	conperc = (annsum / totalsum) * 100  # Percentage contribution to total annual precipitation

    	# Append calculated values to lists
    	pert.append(pert1)  # 95th percentile value
    	maxiv.append(maxi)
    	meanv.append(mean)
    	annsumv.append(annsum)
    	countmean.append(countm)
    	countsum.append(counts)
    	conpercv.append(conperc)

# Reshape the calculated variable to the original spatial pattern and save as xarray DataArray - Example
contripv = np.reshape(np.array(conpercv), (133, 721))
epecontri = xr.DataArray(
	data=contripv,
	coords={'latitude': lats, 'longitude': lons},
	dims=['latitude', 'longitude']
)

# Check the type of the resulting DataArray
type(epecontri)

######################################


######################################
#To find the mean and maximum temperature values during EPE days,

import xarray as xr
import numpy as np

# Initialize empty lists to store calculated values
t2mmn = []   # List for mean temperature during EPE days
t2mmax = []  # List for maximum temperature during EPE days

# Loop through each spatial point
for i in lats:
    for j in lons:
        # Sample data (replace with your actual precipitation and temperature data)
        # ppnsp = daily precipitation values covering the area for the selected time (here, ERA5 hourly data)
        # t2msp = daily temperature values covering the area for the selected time (here, ERA5 hourly data)

        # Select precipitation and temperature values at the current point and filter out low precipitation values
        ppn1 = ppnsp.sel(latitude=i, longitude=j, method='nearest') * 1000
        t2m1v = t2msp.sel(latitude=i, longitude=j, method='nearest')
        ppnd = ppn1[ppn1 > 0.02]

        # Calculate the 95th percentile value
        pert1 = np.percentile(ppnd, 95)

        # Select EPE days based on the 95th percentile
        epe1 = ppnd[ppnd > pert1]
        timev = epe1.time.values

        # Initialize empty lists to store temperature values during EPE days
        t2mepe = []

        # Loop through each EPE day and select temperature values
        for k in timev:
            t2mepe1 = t2m1v.sel(time=k)
            t2mepe.append(t2mepe1)

        # Concatenate temperature values along the time dimension
        t2mvepe1 = xr.concat(t2mepe, dim='time')

        # Calculate mean and maximum temperature during EPE days
        t2mmnv = t2mvepe1.mean(dim='time')
        t2mmaxv = t2mvepe1.max(dim='time')

        # Append calculated values to lists
        t2mmn.append(t2mmnv)
        t2mmax.append(t2mmaxv)

##########################################

# To find the spatial seasonal variability - example: Seasonal total number of EPEs

import xarray as xr
import numpy as np

# Function to split data into seasons
def split_into_seasons(data):
    # Split the data into seasons: JJA, SON, DJF, MAM
    jja = data.sel(time=data['time.season'] == 'JJA')
    son = data.sel(time=data['time.season'] == 'SON')
    djf = data.sel(time=data['time.season'] == 'DJF')
    mam = data.sel(time=data['time.season'] == 'MAM')

    return jja, son, djf, mam

# Initialize empty lists to store seasonal counts
countsvjja = []   # List for counts during JJA
countsvson = []   # List for counts during SON
countsvdjf = []   # List for counts during DJF
countsvmam = []   # List for counts during MAM

# Loop through each spatial point
for i in lats:
    for j in lons:
        # Sample data (replace with your actual precipitation data)
        # ppnsp = daily precipitation values covering the area for the selected time (here, ERA5 hourly data)

        # Select precipitation values at the current point and filter out low precipitation values
        ppn1 = ppnsp.sel(latitude=i, longitude=j, method='nearest') * 1000
        ppnd = ppn1[ppn1 > 0.02]

        # Calculate the 95th percentile value
        pert1 = np.percentile(ppnd, 95)

        # Select EPE days based on the 95th percentile
        epe1 = ppnd[ppnd > pert1]

        # Split EPE data into seasons
        jja_data, son_data, djf_data, mam_data = split_into_seasons(epe1)

        # Calculate seasonal counts
        countsjja = jja_data.groupby('time.year').count().sum().values
        countsson = son_data.groupby('time.year').count().sum().values
        countsdjf = djf_data.groupby('time.year').count().sum().values
        countsmam = mam_data.groupby('time.year').count().sum().values

        # Append calculated values to lists
        countsvjja.append(countsjja)
        countsvson.append(countsson)
        countsvdjf.append(countsdjf)
        countsvmam.append(countsmam)

################################ 

